@@ -1,19 +1,21 @@
-''' Base Transformation class and associated utilities. '''
+"""Base Transformation class and associated utilities. """
 
-import numpy as np
-import pandas as pd
-from bids.utils import listify
+import re
 import warnings
 from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 import itertools
 import inspect
+
+import numpy as np
+import pandas as pd
+
+from bids.utils import listify, convert_JSON
 from bids.variables import SparseRunVariable
-from six import add_metaclass
+from bids.analysis import transformations as pbt
 
 
-@add_metaclass(ABCMeta)
-class Transformation(object):
+class Transformation(metaclass=ABCMeta):
 
     ### Class-level settings ###
     # The following settings govern the way Transformations are applied to the
@@ -87,8 +89,8 @@ class Transformation(object):
         return t.transform()
 
     def _setup(self, collection, variables, *args, **kwargs):
-        ''' Replaces __init__ to set instance attributes because on Python
-        >= 3.3, we can't override both new and init. '''
+        """Replaces __init__ to set instance attributes because on Python
+        >= 3.3, we can't override both new and init. """
         self.collection = collection
         self.variables = listify(variables)
         self.groupby = kwargs.pop('groupby', None)
@@ -101,7 +103,7 @@ class Transformation(object):
         # that operations like densification, alignment, etc. correctly detect
         # all named arguments.
         if args:
-            arg_spec = inspect.getargspec(self._transform)
+            arg_spec = inspect.getfullargspec(self._transform)
             for i, arg_val in enumerate(args):
                 # Skip first two argnames--they're always 'self' and
                 # 'variables'
@@ -109,14 +111,47 @@ class Transformation(object):
 
         self.kwargs = kwargs
 
-        # Expand regex variable names
-        replace_args = self.kwargs.pop('regex_variables', None)
-        if replace_args is not None:
-            self._regex_replace_variables(replace_args)
+        # Expand any detected variable group names or wild cards
+        self._expand_variable_groups()
+        self._expand_variable_names()
+
+    def _expand_variable_groups(self):
+        """ Replace any detected variable groups with the associated lists of
+        variable names.
+        """
+        groups = self.collection.groups
+        variables = [groups[v] if v in groups else [v] for v in self.variables]
+        self.variables = list(itertools.chain(*variables))
+
+    def _expand_variable_names(self):
+        """Filter all available arguments against collection's variables using
+        unix-style pattern matching."""
+        def _replace_arg_values(values):
+            is_iter = isinstance(values, (list, tuple))
+            values = listify(values)
+            result = []
+            # Only try to match strings containing a relevant special character
+            for v in values:
+                if isinstance(v, str) and re.search('[\*\?\[\]]', v):
+                    result.append(self.collection.match_variables(v))
+                else:
+                    result.append([v])
+
+            result = list(itertools.chain(*result))
+            # Don't return a list unless we have to
+            if is_iter or len(result) > 1:
+                return result
+            return result[0]
+
+        # 'variables' is stored separately, so handle it separately
+        self.variables = _replace_arg_values(self.variables)
+
+        for k, arg in self.kwargs.items():
+            self.kwargs[k] = _replace_arg_values(arg)
 
     def _clone_variables(self):
-        ''' Deep copy all variables the transformation touches. This prevents us
-        from unnecessarily overwriting existing variables. '''
+        """Deep copy all variables the transformation touches. This prevents us
+        from unnecessarily overwriting existing variables. """
 
         # Always clone the target variables
         self._variables = {v: self.collection[v].clone()
@@ -136,7 +171,7 @@ class Transformation(object):
                 self._variables[v] = deepcopy(self.collection[v])
 
     def _check_categorical_variables(self):
-        ''' Convert categorical variables to dummy-coded indicators. '''
+        """Convert categorical variables to dummy-coded indicators. """
 
         # Collect variable names to pass through
         pass_thru = []
@@ -173,39 +208,6 @@ class Transformation(object):
                 sr = self.collection.sampling_rate
                 self._variables[v] = var.to_dense(sr)
 
-    def _regex_replace_variables(self, args):
-        ''' For each argument named in args, interpret the values set in the
-        argument as regex patterns to potentially be replaced with variables
-        that match the pattern. '''
-
-        args = listify(args)
-
-        if 'variables' in args:
-            args.remove('variables')
-            variables = True
-        else:
-            variables = False
-
-        # Ensure all keyword arguments user wants to scan are valid
-        missing = set(args) - set(self.kwargs.keys())
-        if missing:
-            raise ValueError("Arguments '%s' specified for regex-based "
-                             "variable name replacement, but were not found "
-                             "among keyword arguments." % missing)
-
-        def _replace_arg_values(names):
-            variables = listify(names)
-            variables = [self.collection.match_variables(c) for c in names]
-            variables = itertools.chain(*variables)
-            return list(set(variables))
-
-        # 'variables' is stored separately, so handle it separately
-        if variables:
-            self.variables = _replace_arg_values(self.variables)
-
-        for arg in args:
-            self.kwargs[arg] = _replace_arg_values(self.kwargs[arg])
-
     def transform(self):
 
         output_passed = not (self.output is None and self.output_prefix is None
@@ -310,12 +312,12 @@ class Transformation(object):
         return col
 
     def _align_variables(self, variables, force=True):
-        ''' Checks whether the specified variables have aligned indexes. This
+        """Checks whether the specified variables have aligned indexes. This
         implies either that all variables are dense, or that all variables are
         sparse and have exactly the same onsets and durations. If variables are
         not aligned and force = True, all variables will be forced to dense
         format in order to ensure alignment.
-        '''
+        """
 
         if self._align is None or self._align == 'none':
             return
@@ -371,3 +373,77 @@ class Transformation(object):
                 _align([c] + align_variables)
         else:
             _align(listify(variables) + align_variables)
+
+
+class TransformerManager(object):
+    """Handles registration and application of transformations to
+    BIDSVariableCollections.
+
+    Parameters
+    ----------
+    default: object
+        A module or other object containing default transformations as
+            attributes. Any named transformation not explicitly registered on
+            the TransformerManager instance is expected to be found here.
+            If None, the PyBIDS transformations module is used.
+    """
+
+    def __init__(self, default=None):
+        self.transformations = {}
+        if default is None:
+            # Default to PyBIDS transformations
+            default = pbt
+        self.default = default
+
+    def _sanitize_name(self, name):
+        """ Replace any invalid/reserved transformation names with acceptable
+        equivalents.
+
+        Parameters
+        ----------
+        name: str
+            The name of the transformation to sanitize.
+        """
+        if name in ('And', 'Or'):
+            name += '_'
+        return name
+
+    def register(self, name, func):
+        """Register a new transformation handler.
+
+        Parameters
+        ----------
+        name : str
+            The name of the transformation to handle.
+        func : callable
+            The callable to invoke when the named transformation is applied.
+        """
+        name = self._sanitize_name(name)
+        self.transformations[name] = func
+
+    def transform(self, collection, transformations):
+        """Apply all transformations to the variables in the collection.
+
+        Parameters
+        ----------
+        collection: BIDSVariableCollection
+            The BIDSVariableCollection containing variables to transform.
+        transformations : list
+            List of transformations to apply.
+        """
+        for t in transformations:
+            t = convert_JSON(t) # make sure all keys are snake case
+            kwargs = dict(t)
+            name = self._sanitize_name(kwargs.pop('name'))
+            cols = kwargs.pop('input', None)
+
+            # Check registered transformations; fall back on default module
+            func = self.transformations.get(name, None)
+            if func is None:
+                if not hasattr(self.default, name):
+                    raise ValueError("No transformation '%s' found: either "
+                                     "explicitly register a handler, or pass a"
+                                     " default module that supports it." % name)
+                func = getattr(self.default, name)
+                func(collection, cols, **kwargs)
+        return collection
