@@ -4,7 +4,6 @@ dataset.
 Parsing functions for generating the MRI data acquisition portion of a
 methods section from a BIDS dataset.
 """
-from __future__ import print_function
 import logging
 from os.path import basename
 
@@ -315,7 +314,7 @@ def fmap_info(metadata, img, config, layout):
         run_dict = {}
         for scan in scans:
             fn = basename(scan)
-            iff_file = [f for f in layout.get(extension='nii.gz') if fn in f.path][0]
+            iff_file = [f for f in layout.get(extension=[".nii", ".nii.gz"]) if fn in f.path][0]
             run_num = int(iff_file.run)
             ty = iff_file.entities['suffix'].upper()
             if ty == 'BOLD':
@@ -439,7 +438,7 @@ def parse_niftis(layout, niftis, subj, config, **kwargs):
 
             if nifti_struct.entities['datatype'] == 'func':
                 if not skip_task.get(nifti_struct.entities['task'], False):
-                    echos = layout.get_echoes(subject=subj, extension='nii.gz',
+                    echos = layout.get_echoes(subject=subj, extension=[".nii", ".nii.gz"],
                                               task=nifti_struct.entities['task'],
                                               **kwargs)
                     n_echos = len(echos)
@@ -447,7 +446,7 @@ def parse_niftis(layout, niftis, subj, config, **kwargs):
                         metadata['EchoTime'] = []
                         for echo in sorted(echos):
                             echo_struct = layout.get(subject=subj, echo=echo,
-                                                     extension='nii.gz',
+                                                     extension=[".nii", ".nii.gz"],
                                                      task=nifti_struct.entities['task'],
                                                      **kwargs)[0]
                             echo_file = echo_struct.path
@@ -457,6 +456,7 @@ def parse_niftis(layout, niftis, subj, config, **kwargs):
                     n_runs = len(layout.get_runs(subject=subj,
                                                  task=nifti_struct.entities['task'],
                                                  **kwargs))
+                    n_runs = max(n_runs, 1)
                     description_list.append(func_info(nifti_struct.entities['task'],
                                                       n_runs, metadata, img,
                                                       config))
@@ -469,7 +469,7 @@ def parse_niftis(layout, niftis, subj, config, **kwargs):
                 description_list.append(anat_info(suffix, metadata, img,
                                                   config))
             elif nifti_struct.entities['datatype'] == 'dwi':
-                bval_file = nii_file.replace('.nii.gz', '.bval')
+                bval_file = nii_file.replace('.nii.gz', '.bval').replace('.nii', '.bval')
                 description_list.append(dwi_info(bval_file, metadata, img,
                                                  config))
             elif nifti_struct.entities['datatype'] == 'fmap':
